// +-------------------------------------------------+
// © 2002-2004 PMB Services / www.sigb.net pmb@sigb.net et contributeurs (voir www.sigb.net)
// +-------------------------------------------------+
// $Id: loanContainer.js,v 1.16 2015-06-12 15:02:09 vtouchard Exp $

define(["dojo/_base/declare", "dojox/grid/DataGrid", "dojo/topic", "dojo/_base/lang", "dojo/dom-construct", "dijit/registry", "dojo/store/Memory", "dojo/data/ObjectStore", "dijit/registry", "app/warningMessage"], function(declare, DataGrid, topic, lang, domConstruct, registry, Memory, ObjectStore, registry, warningMessage) {
    return declare(null, {
        dataGrid : null,
        loanStore : null,
        datasLoan : null,
        antivol_level : 0,
        readingEnabled : true,
        constructor : function() {
            /**
             * Création du store de prêt
             * Instanciation du datagrid
             *
             */
            this.datasLoan = new Array();
            for (var i = 0; i < dojoConfig.bibloto.readerDatas.emprInfos.prets.length; i++) {
                this.datasLoan.push({
                    "title" : dojoConfig.bibloto.readerDatas.emprInfos.prets[i].title,
                    "loan_date" : dojoConfig.bibloto.readerDatas.emprInfos.prets[i].date_pret,
                    'loan_back_date' : dojoConfig.bibloto.readerDatas.emprInfos.prets[i].date_retour,
                    'id' : i,
                });
            }

            this.loanStore = new ObjectStore({
                idProperty : 'title',
                objectStore : new Memory({
                    data : this.datasLoan
                })
            });

            this.dataGrid = new DataGrid({
                store : this.loanStore,
                autoHeight : true,
                query : {
                    id : "*"
                },
                structure : [{
                    name : "Titre",
                    field : "title",
                    editable : false,
                    width : "40%"
                }, {
                    name : "Date prêt",
                    field : "loan_date",
                    editable : false,
                    width : "30%"
                }, {
                    name : "Date retour",
                    field : "loan_back_date",
                    editable : false,
                    width : "30%"
                }]
            }, domConstruct.create('div', {
                id : 'loanDatas',
                style : {
                    height : '300px',
                    width : 'auto'
                }
            }, registry.byId('contentZone').domNode));

            this.dataGrid.startup();

            topic.subscribe('actionZone', lang.hitch(this, this.handleEvents));
            topic.subscribe('warningMessage', lang.hitch(this, this.handleEvents));
        },
        handleEvents : function(evtType, evtArgs) {
            //console.log('evt received in loanContainer', evtType, evtArgs);
            switch(evtType) {
            /**
             * pret evt recu
             */
            case 'explLent':
                this.antivol_level = 0;
                this.addToStore(evtArgs.response, keyboard_cb_enter);
                //console.log(evtArgs.response);
                if (dojoConfig.bibloto.appParams.rfid_activate == 1 && dojoConfig.bibloto.appParams.rfid_security_activate && evtArgs.response.expl_cb != keyboard_cb_enter) {
                    init_rfid_antivol_all_flag = 1;
                    init_rfid_antivol_all_level = 0;
                }
                break;
            case 'explBack':
                this.antivol_level = 1;
                this.popFromStore(evtArgs.response);
                //console.log(evtArgs.response,keyboard_cb_enter);
                if (dojoConfig.bibloto.appParams.rfid_activate == 1 && dojoConfig.bibloto.appParams.rfid_security_activate && evtArgs.response.expl_cb != keyboard_cb_enter) {
                    init_rfid_antivol_all_flag = 1;
                    init_rfid_antivol_all_level = 1;
                }
                break;
            }

        },

        /**
         * Ajout d'un pret au store de pret
         */
        addToStore : function(datas) {
            if (datas.status == 1) {
                var newItm = {
                    title : datas.title,
                    loan_date : this.getCurrDate(),
                    loan_back_date : datas.due_date
                };
                this.loanStore.newItem(newItm);
                this.loanStore.save();
            }
        },

        /**
         * Fonction appelée lorsqu'un exemplaire qui était emprunté est rendu
         */
        popFromStore : function(datas) {

            if (this.loanStore.objectStore.query({
                title : datas.title
            }).length > 0) {
                var idToDelete = this.loanStore.objectStore.query({title:datas.title})[0].id;
                this.loanStore.objectStore.remove(idToDelete);
            }

            this.loanStore.save();
        },

        /**
         * Récuperation de la date courante au format jj/mm/aaaa
         */
        getCurrDate : function() {
            var today = new Date();
            var dd = today.getDate();
            var mm = today.getMonth() + 1;

            var yyyy = today.getFullYear();
            if (dd < 10) {
                dd = '0' + dd;
            }
            if (mm < 10) {
                mm = '0' + mm;
            }
            return dd + '/' + mm + '/' + yyyy;
        }
    });
});
