// +-------------------------------------------------+
// © 2002-2004 PMB Services / www.sigb.net pmb@sigb.net et contributeurs (voir www.sigb.net)
// +-------------------------------------------------+
// $Id: readerDatas.js,v 1.35 2016-04-20 16:23:20 dbellamy Exp $

define(["dojo/_base/declare", "dojo/topic", "dojo/_base/lang", 'app/warningMessage', 'dojo/on'], function(declare, topic, lang, warningMessage, on) {
    return declare(null, {
        emprCb : null,
        emprInfos : null,
        mode : 0, //0 - attente d'une carte lecteur ; 1 - Mode prêt ; 2 - Mode retour ; 3 - Mode attente (pas de lecture effective)
        timer : null,
        currentTempLoan : null,
        constructor : function() {
            /** Abonnement aux évenements provenant des classes "buffer" et "actionZone" **/
            topic.subscribe('buffer', lang.hitch(this, this.handleEvents));
            topic.subscribe('actionZone', lang.hitch(this, this.handleEvents));
            topic.subscribe('warningMessage', lang.hitch(this, this.handleEvents));
            topic.subscribe('reader_rfid', lang.hitch(this, this.handleEvents));
            topic.subscribe('keyboard', lang.hitch(this, this.handleEvents));
            topic.subscribe('workZone', lang.hitch(this, this.handleEvents));
        },
        handleEvents : function(evtType, evtArgs) {
            //console.debug('evt received in READER DATAS', evtType, evtArgs);
            switch (evtType) {
            /** erase: Évenement envoyé par la classe actionZone lors d'un appui sur le bouton retourner à la page d'accueil **/
            case 'erase' :
                this.mode = 0;
                this.eraseTimer();
                break;
            /** eltAdded: Évenement envoyé par la classe buffer lorsqu'un éléments est lu **/
            case 'eltAdded' :
                switch (this.mode) {
                case 0 :
                    this.fetchEmpr(evtArgs.elt);
                    break;
                case 1 :
                    this.callLoanTemp(evtArgs.elt);
                    break;
                case 2 :
                    this.callExplInfos(evtArgs.elt);
                    break;
                }
                break;
                
            /** évenements diffusés par la classe actionZone **/    
            case 'enableFetchEmpr':
                this.mode = 0;
                break;
            case 'enableLoan' :
                dojoConfig.bibloto.appParams.rfid_lastVal = "";
                this.mode = 1;
                break;
            case 'enableBack' :
                dojoConfig.bibloto.appParams.rfid_lastVal = "";
                this.mode = 2;
                break;
            case 'stopListen':
                this.mode = 3;
                break;
            /** évenements diffusés par la classe reader **/
            case 'rfid_expl':
                //console.debug("readerDatas, reader_rfid, rfid_expl:", evtArgs);
                if (evtArgs.cb[0]) {
                    switch (this.mode) {
                    case 1 :
                        //console.debug("readerDatas, rfid_expl, callLoanTemp:", evtArgs);
                        this.callLoanTemp(evtArgs.cb[0]);
                        break;
                    case 2 :
                        //this.callRetour(evtArgs.cb[0]);
                        this.callExplInfos(evtArgs.cb[0]);
                        break;
                    }
                }
                break;
            case 'rfid_noexpl':
                //console.debug("readerDatas, reader_rfid, rfid_noexpl");
                switch (this.mode) {
                case 1 :
                    //console.debug("readerDatas, rfid_noexpl, callLoanTempDel:", evtArgs);
                    this.callLoanTempDel(evtArgs.cb[0]);
                    break;
                case 2 :
                    topic.publish('readerDatas', 'noExplBack');
                    break;
                }
                break;
            case 'rfid_toomanyExpl':
                switch (this.mode) {
                //Trop d'exemplaire en prêt
                case 1 :
                    topic.publish('readerDatas', 'tooManyExplLoan');
                    break;
                case 2 :
                    topic.publish('readerDatas', 'tooManyExplLoan');
                    break;
                }
                // topic.publish('readerDatas', 'tooManyExpl');
                break;
            case 'rfid_empr':
                //console.debug("readerDatas, reader_rfid, rfid_empr");
                switch (this.mode) {
                case 0 :
                    //console.debug("readerDatas, reader_rfid, rfid_empr");
                    this.mode = 3;
                    this.fetchEmpr(evtArgs.cb[0]);
                    break;
                }
                break;
            /** évenement diffusé lors de la validation d'un prêt ou d'un retour par la classe workZone **/
            case 'validateAction':
                //console.debug('MODE READER DATAS', this.mode);
                switch(this.mode) {
                case 1:
                    //Appel du pret definitif
                    this.callLoan(dojoConfig.bibloto.appParams.rfid_lastVal);
                    break;
                case 2:
                    //appêl du retour
                    this.callBack(dojoConfig.bibloto.appParams.rfid_lastVal);
                    break;
                }
                break;
            }
        },
        /** requete json rpc pour effectuer le prêt temporaire**/
        callLoanTemp : function(expl) {
            this.reInitTimer();
            this.mode = 3;
            var emprPretPromise = dojoConfig.bibloto.jsonRpcServer.pmbesSelfServices_self_checkout_bibloto(expl, this.emprCb, 0);
            emprPretPromise.addCallback(lang.hitch(this, function(response) {
                this.currentTempLoan = expl;
                this.mode = 1;
                topic.publish('readerDatas', 'loanTempResponse', {
                    response : response
                });
            }));

        },
        /** requete json rpc pour effacer le prêt temporaire**/
        callLoanTempDel : function(expl) {
            this.reInitTimer();
            this.mode = 3;
            var emprPretPromise = dojoConfig.bibloto.jsonRpcServer.pmbesSelfServices_self_del_temp_pret(expl);
            emprPretPromise.addCallback(lang.hitch(this, function(response) {
                topic.publish('readerDatas', 'loanTempDelResponse', {
                    response : response
                });
                this.mode = 1;
            }));

        },
        /** requete json rpc pour confirmer le prêt**/
        callLoan : function(expl) {
            this.reInitTimer();
            this.mode = 3;
            var emprPretPromise = dojoConfig.bibloto.jsonRpcServer.pmbesSelfServices_self_checkout_bibloto(expl, this.emprCb, 1);
            emprPretPromise.addCallback(lang.hitch(this, function(response) {
                topic.publish('readerDatas', 'loanResponse', {
                    response : response
                });

                this.mode = 1;
            }));

        },

        /** requete json rpc pour effectuer le retour **/
        callBack : function(expl) {
            this.reInitTimer();
            this.mode = 3;
            var emprBackPromise = dojoConfig.bibloto.jsonRpcServer.pmbesSelfServices_self_checkin(expl, null);
            emprBackPromise.addCallback(lang.hitch(this, function(response) {
                topic.publish('readerDatas', 'backResponse', {
                    response : response
                });
                this.mode = 2;
            }));
        },
        /** requête jsonRpc pour récupérer les infos emprunteur **/
        fetchEmpr : function(cb) {
            /**
             * fetch data et callback
             */
        	if(dojoConfig.bibloto.mapping){
        		if(dojoConfig.bibloto.mapping[cb]) {
        			this.emprCb = dojoConfig.bibloto.mapping[cb];
        		} else {
        			this.emprCb='';
        			this.mode = 3;
        			new warningMessage('enableFetchEmpr', 'Lecteur inconnu', 'Erreur');
        		}
        	} else {
        		this.emprCb = cb;
        	}
        	if (this.emprCb) {
        		var infosEmprPromise = dojoConfig.bibloto.jsonRpcServer.pmbesEmpr_fetch_empr(this.emprCb, null);
        		infosEmprPromise.addCallback(lang.hitch(this, this.setEmprInfos));
        	}
        },

        /** si l'emprunteur existe, affectation des données reçues + publication d'un evenement data userLoaded **/
        setEmprInfos : function(response) {
            if (response.status == true) {
                this.emprInfos = response.data;
                topic.publish('readerDatas', 'datasUserLoaded');
                this.initTimer();
            } else {
                this.mode = 3;
                new warningMessage('enableFetchEmpr', response.status_msg, 'Erreur');
                //alert(response.status_msg);
            }
        },
        
        /**
         * Appel de la méthode du webservice retournant les infos de l'exemplaire
         * @param {Object} expl
         */
        callExplInfos : function(expl) {
            this.reInitTimer();
            this.mode = 3;
            var emprExplInfos = dojoConfig.bibloto.jsonRpcServer.pmbesItems_fetch_item_info(expl, null, null);
            emprExplInfos.addCallback(lang.hitch(this, function(response) {
                //console.debug('response infos expl', response);
                this.mode = 2;
                topic.publish('readerDatas', 'gotExplInfos', {
                    response : response
                });

            }));
        },
        
        /**
         * Initialisation du timer
         */
        initTimer : function() {
            this.reInitTimer();
            this.mousemoveEvent = on(document.body, 'mousemove', lang.hitch(this, function() {
                this.reInitTimer();
            }));
            this.keypressEvent = on(document.body, 'keypress', lang.hitch(this, function() {
                this.reInitTimer();
            }));
        },
        /**
         * Réinitialisation du timer
         */
        reInitTimer : function() {
            if (this.timer) {
                clearTimeout(this.timer);
            }
            this.timer = setTimeout(lang.hitch(this, function() {
                //console.log('erase published');
                topic.publish('actionZone', 'erase');
            }), (parseInt(dojoConfig.bibloto.appParams.timeout_disconnect) * 1000));
        },
        
        /**
         * Suppression du timer (au retour à l'écran de login') 
         */
        eraseTimer : function() {
            this.mousemoveEvent.remove();
            this.keypressEvent.remove();
            clearTimeout(this.timer);
            this.timer = null;
        },
    });
});
